package gov.nih.nlm.nls.lexCheck.Cat.Pron;
import java.util.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.CkLib.*;
/*****************************************************************************
* This class checks filler for syntactical category of pronoun.
*
* <p><b>History:</b>
* <ul>
* <li>SCR-58: chlu, 11/20/18, add a new field class_type - check syntax
* </ul>
*
* @author NLM LVG Development Team
*
* @version    V-2019
****************************************************************************/
public class CheckPron
{
    // public method
    /**
    * Check a line to see if it is a legal filler.
    *
    * @param   lineObject  the java object of a input line
    * @param   printFlag   the flag for print out check result
    * @param   catSt       the state of category
    * @param   lexObj   the current object of lexical record
    * @param   debugFlag   the flag for print out debug information
    *
    * @return  true or false for legal and illegal filler.
    */
    public static boolean Check(LineObject lineObject, boolean printFlag,
        CheckSt catSt, LexRecord lexObj, boolean debugFlag)
    {
        boolean flag = true;
        int curSt = catSt.GetCurState();
        // reset the state if it is first time
        if(curSt == CheckSt.CHECK_CAT_ENTRY)
        {
            curSt = CheckSt.CHECK_PRON_VARIANTS;
            catSt.SetCurState(curSt);
        }
        switch(curSt)
        {
            case CheckSt.CHECK_PRON_VARIANTS:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkVariants_, new UpdatePronVariants(), 
                    CheckCode.FQ_ONE_MANY, true);
                PrintStep(CheckSt.CHECK_PRON_VARIANTS, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_PRON_GENDER:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkGender_, new UpdatePronGender(), 
                    CheckCode.FQ_ZERO_ONE, true);
                PrintStep(CheckSt.CHECK_PRON_GENDER, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_PRON_INTERROGATIVE:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkInterrogative_, new UpdatePronInterrogative(), 
                    CheckCode.FQ_ZERO_ONE_WHOLE_LINE, false);
                PrintStep(CheckSt.CHECK_PRON_INTERROGATIVE, debugFlag, 
                    lineObject.GetLine());
                break;
            case CheckSt.CHECK_PRON_TYPE:
                flag = CheckCode.Check(lineObject, printFlag, catSt, lexObj,
                    checkType_, new UpdatePronType(), 
                    CheckCode.FQ_ZERO_MANY, false);
                PrintStep(CheckSt.CHECK_PRON_TYPE, debugFlag, 
                    lineObject.GetLine());
                break;
        }
        return flag;
    }
    // private method
    private static void PrintStep(int state, boolean debugFlag, String line)
    {
        if(debugFlag == true)
        {
            switch(state)
            {
                case CheckSt.CHECK_PRON_VARIANTS:
                    System.out.println("-- Checked Pron Variants: '" + line 
                        + "'");
                    break;
                case CheckSt.CHECK_PRON_GENDER:
                    System.out.println("-- Checked Pron Gender: '" + line 
                        + "'");
                    break;
                case CheckSt.CHECK_PRON_INTERROGATIVE:
                    System.out.println("-- Checked Pron Interrogative: '" 
                        + line + "'");
                    break;
                case CheckSt.CHECK_PRON_TYPE:
                    System.out.println("-- Checked Pron Type: '" + line + "'");
                    break;
            }
        }
    }
    // data member
    private static CheckObject checkVariants_ = null;
    private static HashSet<String> variantsNextStartStrs_ 
        = new HashSet<String>(8);
    private static CheckObject checkGender_ = null;
    private static HashSet<String> genderNextStartStrs_ 
        = new HashSet<String>(7);
    private static CheckObject checkInterrogative_ = null;
    private static HashSet<String> interrogativeNextStartStrs_ 
        = new HashSet<String>(6);
    private static CheckObject checkType_ = null;
    private static HashSet<String> typeNextStartStrs_ = new HashSet<String>(5);
    static
    {
        // slot variants=
        variantsNextStartStrs_.add("\tgender=");
        variantsNextStartStrs_.add("\tinterrogative");
        variantsNextStartStrs_.add("\ttype=");
        variantsNextStartStrs_.add("\tacronym_of=");
        variantsNextStartStrs_.add("\tabbreviation_of=");
        variantsNextStartStrs_.add("\tclass_type=");
        variantsNextStartStrs_.add("annotation=");
        variantsNextStartStrs_.add("signature=");
        variantsNextStartStrs_.add("}");
        checkVariants_ = new CheckObject("\tvariants=", 
            ErrMsg.PRON_VAR_SLOT_ERR, ErrMsg.PRON_VAR_FILLER_ERR, 
            CheckSt.CHECK_PRON_GENDER, variantsNextStartStrs_,
            new CheckFormatPronVariants());
        // slot gender=
        genderNextStartStrs_.add("\tinterrogative");
        genderNextStartStrs_.add("\ttype=");
        genderNextStartStrs_.add("\tacronym_of=");
        genderNextStartStrs_.add("\tabbreviation_of=");
        genderNextStartStrs_.add("\tclass_type=");
        genderNextStartStrs_.add("annotation=");
        genderNextStartStrs_.add("signature=");
        genderNextStartStrs_.add("}");
        checkGender_ = new CheckObject("\tgender=", 
            ErrMsg.PRON_GENDER_SLOT_ERR, ErrMsg.PRON_GENDER_FILLER_ERR,
            CheckSt.CHECK_PRON_INTERROGATIVE, genderNextStartStrs_, 
            new CheckFormatPronGender());
        // slot interrogative
        interrogativeNextStartStrs_.add("\ttype=");
        interrogativeNextStartStrs_.add("\tacronym_of=");
        interrogativeNextStartStrs_.add("\tabbreviation_of=");
        interrogativeNextStartStrs_.add("\tclass_type=");
        interrogativeNextStartStrs_.add("annotation=");
        interrogativeNextStartStrs_.add("signature=");
        interrogativeNextStartStrs_.add("}");
        checkInterrogative_ = new CheckObject("\tinterrogative", 
            ErrMsg.PRON_INTERROGATIVE_SLOT_ERR, ErrMsg.NONE,
            CheckSt.CHECK_PRON_TYPE, interrogativeNextStartStrs_, null);
        // slot type=
        typeNextStartStrs_.add("\tacronym_of=");
        typeNextStartStrs_.add("\tabbreviation_of=");
        typeNextStartStrs_.add("\tclass_type=");
        typeNextStartStrs_.add("annotation=");
        typeNextStartStrs_.add("signature=");
        typeNextStartStrs_.add("}");
        checkType_ = new CheckObject("\ttype=", 
            ErrMsg.PRON_TYPE_SLOT_ERR, ErrMsg.PRON_TYPE_FILLER_ERR,
            CheckSt.CHECK_ACRONYM, typeNextStartStrs_, 
            new CheckFormatPronType());
    }
}
