package gov.nih.nlm.nls.lexCheck.CheckCont;
import java.util.*;
import java.io.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
/*****************************************************************************
* This class provides a Java class to check cross-ref EUI on class type.
*
* <p><b>History:</b>
* <ul>
* <li>SCR-58: chlu, 11/29/18, add a new field class_type - check cross-reference
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class CrossCheckClassTypeEui
{
    // private constructor
    private CrossCheckClassTypeEui()
    {
    }
    // lexRecord is automatically fixed
    public static boolean Check(LexRecord lexRecord, 
        HashSet<String> notBaseFormSet) 
    {
        boolean validFlag = true;
        Vector<String> ctList = lexRecord.GetClassTypes();    // class types
        String ctCat = lexRecord.GetCategory();
		// go through all class types list from the input lexRecord
        for(int i = 0; i < ctList.size(); i++)
        {
            String classType = ctList.elementAt(i);

			// syntax and contents should be checked before cross-ref check
			// check the type if it is informal
			if(classType.startsWith("informal|") == true)
			{
				// tokenize class types
				String[] args = classType.split("\\|");
				String ctBase = args[1];	// class type base
				String crCit = args[2];	// cross-ref citation, could be base

				// get eui in Lexicon by citation and cat
				// get all possible EUIs by citation and cat
				String citCat = crCit + LexRecordUtil.FS_STR + ctCat;
				HashSet<String> euisByCitCat = 
					CrossCheckDupLexRecords.GetEuisByCitCat(citCat);

				// has no cross-ref EUI in the Lexicon
				if(args.length == 3)
				{
					// single EUI: 2.6 missing EUI
					// tested: class_type=informal|base|crCit
					if((euisByCitCat != null)
					&& (euisByCitCat.size() == 1))    // single EUI in Lexicon
					{
						Vector<String> euiList 
							= new Vector<String>(euisByCitCat);
						String newEui = euiList.elementAt(0);
						// tested: class_type=informal|base|crCit
						// no crEui => Add it in (belong to args=2)
						// auto-fix
						String fixStr = "informal|" + ctBase 
							+ LexRecordUtil.FS_STR + crCit 
							+ LexRecordUtil.FS_STR + newEui;
						ctList.set(i, fixStr);
						// sent out auto fix msg
						validFlag = false;
						ErrMsgUtilLexicon.AddContentErrMsg(
							LexRecordUtil.CROSS_CHK_CT,
							ErrMsgUtilLexicon.ERR_MISS_EUI, 
							classType + " - " + newEui, lexRecord);
					}
					// multiple EUIs: 2.8 missing EUIs
					// tested: class_type=informal|base|crCit, 
					// where crCit|ctCata are multiply exist in Lexicon
					else if((euisByCitCat != null)
					&& (euisByCitCat.size() > 1))  // multiple EUIs in Lexicon
					{
						Vector<String> euiList 
							= new Vector<String>(euisByCitCat);
						// sent out msg
						validFlag = false;
						ErrMsgUtilLexicon.AddContentErrMsg(
							LexRecordUtil.CROSS_CHK_CT,
							ErrMsgUtilLexicon.ERR_MISS_EUIS, 
							classType + " - " + euiList, lexRecord);
					}
					// 2.13 new EUI for candidates
					// tested: class_type=informal|base|newCrCit
					else if(euisByCitCat == null)    // no eui in Lexicon
					{
						// no EUI matches: send msg to add to to-do list
						// send errMsg only if crCit is not one of notBaseForm
						if(notBaseFormSet.contains(citCat) == false)
						{
							validFlag = false;
							ErrMsgUtilLexicon.AddContentErrMsg(
								LexRecordUtil.CROSS_CHK_CT,
								ErrMsgUtilLexicon.ERR_NEW_EUI, 
								classType + " - New", lexRecord);
						}
					}
					else
					// not suppose to happen if all miss EUI are fixed
					{
						// new citation without EUI, could be used for new rec
						if(notBaseFormSet.contains(citCat) == false)
						{
							validFlag = false;
							ErrMsgUtilLexicon.AddContentErrMsg(
								LexRecordUtil.CROSS_CHK_CT,
								ErrMsgUtilLexicon.ERR_NEW_EUI, 
								classType + " - New2", lexRecord);
						}
					}
				}
				// has cross-ref EUI in the Lexicon
				else if(args.length == 4)
				{
					String crEui = args[3];	// cross-ref Eui 
					// get citation in Lexicon by base, Eui, and cat
					// the input crCit could be a spVar (base)
					String crCit1 = CheckEui.GetCitByEuiBase(
						crCit, crEui, ctCat);

					// These checks are better to be proceed in the same order
					// 2.3. no EUI: the crEui is not in Lexicon
					// tested: yyclass_type=informal|base|wrongCit|wrongEUI
					// also triggle 2.4 and 2.5 wrong citations
					if(euisByCitCat == null)    // no eui in Lexicon
					{
						// auto-fix
						String fixStr = "informal|" + ctBase 
							+ LexRecordUtil.FS_STR + crCit;
						ctList.set(i, fixStr);
						// sent auto-fix msg
						validFlag = false;
						ErrMsgUtilLexicon.AddContentErrMsg(
							LexRecordUtil.CROSS_CHK_CT,
							ErrMsgUtilLexicon.ERR_NO_EUI, 
							classType + " - None", lexRecord);
					}
					// tested: class_type=informal|base|crCit|wrongEUI
					// This error also trigger 2.7 wrong EUI or 2.9 wrong EUIs
					else if((crCit1 == null) // no cit found by crCit|crEui 
					&& (notBaseFormSet.contains(citCat) == false)) 
					// not invalid base
					{
						// auto-fix: remove EUI
						String fixStr = "informal|" + ctBase 
							+ LexRecordUtil.FS_STR + crCit;
						ctList.set(i, fixStr);
						// sent auto-fix msg
						validFlag = false;
						ErrMsgUtilLexicon.AddContentErrMsg(
							LexRecordUtil.CROSS_CHK_CT, 
							ErrMsgUtilLexicon.ERR_NO_EUI, 
							classType + " - None", lexRecord);
					}

					// 2.4 & 2.5 wrong citations
					// tested: class_type=informal|base|crSpVar|wrongEUI
					// also triglle 2.3.1 no EUI
					if((crCit1 != null) // cit found by crCit|crEui 
					&& (crCit1.equals(crCit) == false))
					// crCit is a base (spVar), not the citation
					{
						validFlag = false;
						// auto-fix: change cit
						String fixStr = "informal|" + ctBase 
							+ LexRecordUtil.FS_STR + crCit1 
							+ LexRecordUtil.FS_STR + crEui;
						// already exist after changing spVar to citation
						if(ctList.contains(fixStr) == true)
						{
							// auto-fix: remove
							ctList.remove(i);
							// sent auto-fix msg
							ErrMsgUtilLexicon.AddContentErrMsg(
								LexRecordUtil.CROSS_CHK_CT,
								ErrMsgUtilLexicon.ERR_WRONG_CIT_SPVAR_DUP,
								classType + " - wrong cit: [" + crCit
								+ "], is spVar, replaced by cit: [" + crCit1
								+ "] becomes duplicates", lexRecord);
						}
						else	// not exist, replace base/spVar by citation
						{
							// auto-fix: use correct citation
							ctList.set(i, fixStr);
							// sent auto-fix msg
							ErrMsgUtilLexicon.AddContentErrMsg(
								LexRecordUtil.CROSS_CHK_CT, 
								ErrMsgUtilLexicon.ERR_WRONG_CIT_SPVAR, 
								classType + " - wrong cit: [" + crCit
								+ "], is spVar, replaced by cit: [" + crCit1 
								+ "]", lexRecord);
						}
					}

					// single EUI: 2.7 wrong EUI
					// tested: class_type=informal|base|crCit|wrongEUI
					// this error also trigger 2.3.2
					if((euisByCitCat != null)
					&& (euisByCitCat.size() == 1))    // single EUI in Lexicon
					{
						Vector<String> euiList 
							= new Vector<String>(euisByCitCat);
						String newEui = euiList.elementAt(0);
						// tested: class_type=informal|base|crCit|wrongEUI
						// wrong EUI
						if(euisByCitCat.contains(crEui) == false)
						{
							// sent warning msg
							validFlag = false;
							ErrMsgUtilLexicon.AddContentErrMsg(
								LexRecordUtil.CROSS_CHK_CT,
								ErrMsgUtilLexicon.ERR_WRONG_EUI, 
								classType + " - " + newEui, lexRecord);
						}
					}

					// multiple EUIs: 2.9 wrong EUIs
					// tested: class_type=informal|base|crCit|wrongEUI
					// where crCit|ctCat has mulitple EUIs
					if((euisByCitCat != null)
					&& (euisByCitCat.size() > 1))  // multiple EUIs in Lexicon
					{
						Vector<String> euiList 
							= new Vector<String>(euisByCitCat);
						// no eui even multiple matched EUI found
						if(euisByCitCat.contains(crEui) == false)
						{
							// sent warning msg
							validFlag = false;
							ErrMsgUtilLexicon.AddContentErrMsg(
								LexRecordUtil.CROSS_CHK_CT,
								ErrMsgUtilLexicon.ERR_WRONG_EUIS, 
								classType + " - " + euiList, lexRecord);
						}
					}
				}
			}
		}
        return validFlag;
    }
    // data member
}
