package gov.nih.nlm.nls.lexCheck.CheckCont;
import java.util.*;
import java.io.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
/*****************************************************************************
* This class provides a Java class to cross check the content of a lexicon.
* First, it uses lexCheck to check syntax.
* Second, it uses CheckLexRecord to check the content of all lexical records.
* Third, it checks cross-ref acorss the entire lexicon.
*
* <p><b>History:</b>
* <ul>
* <li>SCR-58: chlu, 11/29/18, add a new field class_type - check cross-reference
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class CrossCheckLexRecords
{
    // private constructor
    private CrossCheckLexRecords()
    {
    }
    // lexRecord is automatically fixed
	// dupRecExpList: a hashTable includes key: base, value: EUIs
	// notBaseFormSet: a set includes not base form (invalid LMWs)
    public static void CheckFromFile(String inFile, String outFile, 
        boolean verbose, Hashtable<String, HashSet<String>> dupRecExpList,
        HashSet<String> notBaseFormSet)
    {
        try
        {
            // load all lexical records from the input file    
            Vector<LexRecord> lexRecords 
                = ToJavaObjApi.ToJavaObjsFromTextFile(inFile);
            BufferedWriter out = new BufferedWriter(new OutputStreamWriter(
                new FileOutputStream(outFile), "UTF-8"));
            BufferedWriter dupOut = new BufferedWriter(new OutputStreamWriter(
                new FileOutputStream(outFile+".dupRec"), "UTF-8"));
            // check contents for all lexRecords globally, cross-ref
            // out is used for auto-corrected lexRecords
            CheckLexRecords(lexRecords, out, dupOut, verbose, dupRecExpList,
                notBaseFormSet);
            out.close();
            dupOut.close();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }
    // check contents for all lexRecords globally, cross-ref
    public static void CheckLexRecords(Vector<LexRecord> lexRecords,
        BufferedWriter out, BufferedWriter dupOut, boolean verbose,
        Hashtable<String, HashSet<String>> dupRecExpList,
        HashSet<String> notBaseFormSet) throws IOException
    {
        // load global data to hashtable
        if(verbose == true)
        {
            System.out.println("===== Check Lexicon Cross-Ref Contents =====");
        }
        // reset errMsg_
        ErrMsgUtilLexicon.ResetErrMsg();
        boolean validFlag = true;
        // 1. check EUIs
        boolean dupEuiFlag = CrossCheckDupEuis.Check(lexRecords); 
        // 2. check possible duplicate records by citation|cat
        boolean dupRecFlag = CrossCheckDupLexRecords.Check(lexRecords, 
            dupRecExpList, dupOut, verbose);
        validFlag = dupEuiFlag && dupRecFlag;

        // 3. check and fix cross-ref EUIs for each records
		CheckEui.Init(lexRecords);
        int recSize = lexRecords.size();
        for(int i = 0; i < recSize; i++)
        {
            LexRecord lexRecord = lexRecords.elementAt(i);
            // check nominalization
            boolean nomFlag = CrossCheckNomEui.Check(lexRecord, notBaseFormSet);
            
            // check abbreviation
            boolean abbFlag = CrossCheckAbbEui.Check(lexRecord, notBaseFormSet);
            // check acronym
            boolean acrFlag = CrossCheckAcrEui.Check(lexRecord, notBaseFormSet);

			// check class type = informal
            boolean ctFlag = CrossCheckClassTypeEui.Check(lexRecord, 
				notBaseFormSet);

			// update final valid flag
            validFlag = validFlag && nomFlag && abbFlag && acrFlag && ctFlag;
            // print out fixed records
            if(out != null)
            {
                String text = lexRecord.GetText();
                out.write(text);
            }
        }

        // 4. Cross check symmetric nom
        boolean symFlag = CrossCheckNomSym.Check(lexRecords);
        validFlag = symFlag && validFlag;
        
        // print error message
        if(validFlag == false)
        {
            System.out.println(ErrMsgUtilLexicon.GetErrMsg());
        }
        // send out final message
        System.out.println("----- Total lexRecords checked: " + recSize);
        System.out.println("----- cross-ref content error type stats -----");
        System.out.println(ErrMsgUtilLexicon.GetErrStats());
    }
    // private method
    // data member
}
