package gov.nih.nlm.nls.lexCheck.Lib;
import java.util.*;
/*****************************************************************************
* This class provides a Java class of inflectional variants for a lexical 
* record.
*
* <p><b>History:</b>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class InflVarsAndAgreements
{
    /**
    * Creates a default InflVarsAndAgreements object specifing lexRecord.
    * The inflectional rule is included in a lexical record.  These rules
    * are used to generate the inflectional variants.
    *
    * @param   lexRecord   the lexical record contain inflectional variants
    */
    public InflVarsAndAgreements(LexRecord lexRecord)
    {
        lexRecord_ = lexRecord;
        String base = lexRecord.GetBase();
        String cit = lexRecord.GetBase();
        String eui = lexRecord.GetEui();
        String cat = lexRecord.GetCategory();
        // get inflVar & agreement for it's self (base)
        GetInflVarsAndAgreements(base, eui, cat, cit);
        // get inflVar & agreement for spelling variants
        Vector<String> sv = lexRecord.GetSpellingVars();
        for(int i = 0; i < sv.size(); i++)
        {
            String svBase = sv.elementAt(i);
            GetInflVarsAndAgreements(svBase, eui, cat, cit);
        }
    }
    // public method
    /**
    * Get the xml format of the inflectional variants 
    *
    * @param   lexRecord   the lexical record contain inflectional variants
	*
	* @return  string representation of the input lexical record
    */
    public static String GetXml(LexRecord lexRecord)
    {
        boolean convertFlag = true;
        InflVarsAndAgreements inflVars = new InflVarsAndAgreements(lexRecord);
        String out = new String();
        Vector<InflVar> inflValues = inflVars.GetInflValues();
        for(int i = 0; i < inflValues.size(); i++)
        {
            InflVar inflectionVar = inflValues.elementAt(i);
            String typeAttribute = new String();
            if(inflectionVar.GetType() != null)
            {
                typeAttribute = " type=\"" + inflectionVar.GetType() + "\"";
            }
            // attribute out by alphabetical order
            String startTag = "<inflVars cat=\"" + inflectionVar.GetCat() 
                + "\" cit=\"" + Convert.ToNumericEntity(inflectionVar.GetCit())
                + "\" eui=\"" + inflectionVar.GetEui() 
                + "\" infl=\"" + inflectionVar.GetInflection()
                + "\"" + typeAttribute  
                + " unInfl=\"" 
                + Convert.ToNumericEntity(inflectionVar.GetUnInfl())
                + "\">";
            out = XmlLib.AddToXml(out, startTag, "</inflVars>", 
                inflectionVar.GetVar(), 2, convertFlag); 
        }
        return out;
    }
    /**
    * Get the collection of inflectional variants
    *
    * @return  a Vector with inflectional varaint elements.
    */
    public Vector<InflVar> GetInflValues()
    {
        return inflValues_;
    }
    /**
    * Get the collection of agreements
    *
    * @return  a Vector with agreement elements.
    */
    public Vector<Agreement> GetAgreementValues()
    {
        return agrValues_;
    }
    // private methods
    private void GetInflVarsAndAgreements(String unInfl, String eui, 
        String cat, String cit)
    {
        if(cat.equals("verb"))
        {
            GetVerbInflVarsAndAgreements(unInfl, eui, cat, cit);
        }
        else if(cat.equals("noun"))
        {
            GetNounInflVarsAndAgreements(unInfl, eui, cat, cit);
        }
        else if(cat.equals("adj"))
        {
            GetAdjInflVarsAndAgreements(unInfl, eui, cat, cit);
        }
        else if(cat.equals("adv"))
        {
            GetAdvInflVarsAndAgreements(unInfl, eui, cat, cit);
        }
        else if(cat.equals("aux"))
        {
            GetAuxInflVarsAndAgreements(unInfl, eui, cat, cit);
        }
        else if(cat.equals("modal"))
        {
            GetModalInflVarsAndAgreements(unInfl, eui, cat, cit);
        }
        else if(cat.equals("pron"))
        {
            GetPronInflVarsAndAgreements(unInfl, eui, cat, cit);
        }
        else if(cat.equals("det"))
        {
            GetDetInflVarsAndAgreements(unInfl, eui, cat, cit);
        }
        else if(cat.equals("prep"))
        {
            GetPrepInflVarsAndAgreements(unInfl, eui, cat, cit);
        }
        else if(cat.equals("compl"))
        {
            GetComplInflVarsAndAgreements(unInfl, eui, cat, cit);
        }
        else if(cat.equals("conj"))
        {
            GetConjInflVarsAndAgreements(unInfl, eui, cat, cit);
        }
    }
    private void GetVerbInflVarsAndAgreements(String unInfl, String eui, 
        String cat, String cit)
    {
        // base & singular & present (1p_2_3p)
        inflValues_.addElement(new InflVar(eui, unInfl, "base", unInfl,
            cat, cit));
        // TBD: should be "pres123p"
        inflValues_.addElement(new InflVar(eui, unInfl, "pres1p23p", 
            unInfl, cat, cit));
        agrValues_.addElement(new Agreement(eui, unInfl, cat, 
            "pres(fst_sing,fst_plur,thr_plur,second)", unInfl, cit));
        InflVar infinitive = 
            new InflVar(eui, unInfl, "infinitive", unInfl, cat, cit);
        Agreement agrInf = new Agreement(eui, unInfl, cat, "infinitive", 
            unInfl, cit);
        // go through variants list
        Vector<String> variants 
            = lexRecord_.GetCatEntry().GetVerbEntry().GetVariants();
        String inflVar = new String();
        for(int i = 0; i < variants.size(); i++)
        {
            String variant = variants.elementAt(i);
            if(variant.startsWith("regd"))
            {
                if(inflValues_.contains(infinitive) == false)
                {
                    inflValues_.addElement(infinitive);
                }
                if(agrValues_.contains(agrInf) == false)
                {
                    agrValues_.addElement(agrInf);
                }
                String type = "regd";
                char lastChar = GetLastChar(unInfl);
                char last2Char = GetLast2Char(unInfl);
                String lastCharStr = new Character(lastChar).toString();
                String last2CharStr = new Character(last2Char).toString();
                // check if the base form ends with VC
                if((consonants_.contains(lastCharStr) == false)
                || (vowels_.contains(last2CharStr) == false))
                {
                    System.err.println("** Err@regd violation: " + eui + "|"
                        + unInfl + "|" + cat);
                }
                InflVar pres3s = null;
                // present 3 ps
                if((unInfl.endsWith("s"))
                || (unInfl.endsWith("z"))
                || (unInfl.endsWith("x"))
                || (unInfl.endsWith("ch"))
                || (unInfl.endsWith("sh")))
                {
                    inflVar = unInfl + "es";
                    pres3s = new InflVar(eui, unInfl, type, "pres3s", inflVar, 
                        cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar,
                        cat, "pres(thr_sing)", unInfl, cit));
                }
                else if(unInfl.endsWith("ie"))
                {
                    inflVar = unInfl + "s";
                    pres3s = new InflVar(eui, unInfl, type, "pres3s", inflVar, 
                        cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar,
                        cat, "pres(thr_sing)", unInfl, cit));
                }
                else if((unInfl.endsWith("ee"))
                || (unInfl.endsWith("oe"))
                || (unInfl.endsWith("ye")))
                {
                    inflVar = unInfl + "s";
                    pres3s = new InflVar(eui, unInfl, 
                        type, "pres3s", inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar,
                        cat, "pres(thr_sing)", unInfl, cit));
                }
                else if((unInfl.endsWith("y"))
                && (consonants_.contains(last2CharStr)))
                {
                    inflVar = unInfl.substring(0, unInfl.length()-1) + "ies";
                    pres3s = new InflVar(eui, unInfl, type, "pres3s", inflVar,
                        cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar,
                        cat, "pres(thr_sing)", unInfl, cit));
                }
                else if((unInfl.endsWith("e"))
                && (eioySets_.contains(last2CharStr) == false))
                {
                    inflVar = unInfl + "s";
                    pres3s = new InflVar(eui, unInfl, type, "pres3s", 
                        inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar,
                        cat, "pres(thr_sing)", unInfl, cit));
                }
                else
                {
                    inflVar = unInfl + "s";
                    pres3s = new InflVar(eui, unInfl, type, "pres3s", 
                        inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar,
                        cat, "pres(thr_sing)", unInfl, cit));
                }
                pres3s.SetUnique(IsUnique(inflValues_, pres3s));
                inflValues_.addElement(pres3s);
                // past
                inflVar = unInfl + lastChar + "ed";
                inflValues_.addElement(new InflVar(eui, unInfl, type, 
                    "past", inflVar, cat, cit));
                agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                    "past", unInfl, cit));
                // past participle
                inflVar = unInfl + lastChar + "ed";
                inflValues_.addElement(new InflVar(eui, unInfl, type, 
                    "pastPart", inflVar, cat, cit));
                agrValues_.addElement(new Agreement(eui, inflVar,
                    cat, "past_part", unInfl, cit));
                // present participle
                inflVar = unInfl + lastChar + "ing";
                inflValues_.addElement(new InflVar(eui, unInfl, type, 
                    "presPart", inflVar, cat, cit));
                agrValues_.addElement(new Agreement(eui, inflVar,
                    cat, "pres_part", unInfl, cit));
            }
            else if(variant.startsWith("reg"))
            {
                if(inflValues_.contains(infinitive) == false)
                {
                    inflValues_.addElement(infinitive);
                }
                if(agrValues_.contains(agrInf) == false)
                {
                    agrValues_.addElement(agrInf);
                }
                String type = "reg";
                char lastChar = GetLastChar(unInfl);
                char last2Char = GetLast2Char(unInfl);
                String last2CharStr = new Character(last2Char).toString();
                InflVar pres3s = null;
                InflVar past = null;
                InflVar pastPart = null;
                InflVar presPart = null;
                if((unInfl.endsWith("s"))
                || (unInfl.endsWith("z"))
                || (unInfl.endsWith("x"))
                || (unInfl.endsWith("ch"))
                || (unInfl.endsWith("sh")))
                {
                    // present 3 ps
                    inflVar = unInfl + "es";
                    pres3s = new InflVar(eui, unInfl, 
                        type, "pres3s", inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "pres(thr_sing)", unInfl, cit));
                    // past
                    inflVar = unInfl + "ed";
                    past = new InflVar(eui, unInfl, 
                        type, "past", inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "past", unInfl, cit));
                    // past participle
                    inflVar = unInfl + "ed";
                    pastPart = new InflVar(eui, unInfl, 
                        type, "pastPart", inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "past_part", unInfl, cit));
                    // present participle
                    inflVar = unInfl + "ing";
                    presPart = new InflVar(eui, unInfl, 
                        type, "presPart", inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "pres_part", unInfl, cit));
                }
                else if(unInfl.endsWith("ie"))
                {
                    inflVar = unInfl + "s";
                    // present 3 ps
                    pres3s = new InflVar(eui, unInfl, 
                        type, "pres3s", inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "pres(thr_sing)", unInfl, cit));
                    // past
                    inflVar = unInfl + "d";
                    past = new InflVar(eui, unInfl, 
                        type, "past", inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "past", unInfl, cit));
                    // past participle
                    inflVar = unInfl + "d";
                    pastPart = new InflVar(eui, unInfl, 
                        type, "pastPart", inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "past_part", unInfl, cit));
                    // present participle
                    inflVar = unInfl.substring(0, unInfl.length()-2) + "ying";
                    presPart = new InflVar(eui, unInfl, type, 
                        "presPart", inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "pres_part", unInfl, cit));
                }
                else if((unInfl.endsWith("ee"))
                || (unInfl.endsWith("oe"))
                || (unInfl.endsWith("ye")))
                {
                    // present 3 ps
                    inflVar = unInfl + "s";
                    pres3s = new InflVar(eui, unInfl, 
                        type, "pres3s", inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "pres(thr_sing)", unInfl, cit));
                    // past
                    inflVar = unInfl + "d";
                    past = new InflVar(eui, unInfl, 
                        type, "past", inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "past", unInfl, cit));
                    // past participle
                    inflVar = unInfl + "d";
                    pastPart = new InflVar(eui, unInfl, 
                        type, "pastPart", inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "past_part", unInfl, cit));
                    // present participle
                    inflVar = unInfl + "ing";
                    presPart = new InflVar(eui, unInfl, 
                        type, "presPart", inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "pres_part", unInfl, cit));
                }
                else if((unInfl.endsWith("y"))
                && (consonants_.contains(last2CharStr)))
                {
                    // present 3 ps
                    inflVar = unInfl.substring(0, unInfl.length()-1) + "ies";
                    pres3s = new InflVar(eui, unInfl, type, "pres3s", 
                        inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "pres(thr_sing)", unInfl, cit));
                    // past
                    inflVar = unInfl.substring(0, unInfl.length()-1) + "ied";
                    past = new InflVar(eui, unInfl, type, "past", 
                        inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "past", unInfl, cit));
                    // past participle
                    inflVar = unInfl.substring(0, unInfl.length()-1) + "ied";
                    pastPart = new InflVar(eui, unInfl, type, "pastPart",
                        inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "past_part", unInfl, cit));
                    // present participle
                    inflVar = unInfl + "ing";
                    presPart = new InflVar(eui, unInfl, 
                        type, "presPart", inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "pres_part", unInfl, cit));
                }
                else if((unInfl.endsWith("e"))
                && (eioySets_.contains(last2CharStr) == false))
                {
                    // present 3 ps
                    inflVar = unInfl + "s";
                    pres3s = new InflVar(eui, unInfl, type, "pres3s", 
                        inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "pres(thr_sing)", unInfl, cit));
                    // past
                    inflVar = unInfl + "d";
                    past = new InflVar(eui, unInfl, type, "past", 
                        inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "past", unInfl, cit));
                    // past participle
                    inflVar = unInfl + "d";
                    pastPart = new InflVar(eui, unInfl, type, "pastPart", 
                        inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "past_part", unInfl, cit));
                    // present participle
                    inflVar = unInfl.substring(0, unInfl.length()-1) + "ing";
                    presPart = new InflVar(eui, unInfl, type, "presPart",
                        inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "pres_part", unInfl, cit));
                }
                else
                {
                    // present 3 ps
                    inflVar = unInfl + "s";
                    pres3s = new InflVar(eui, unInfl, type, "pres3s", 
                        inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "pres(thr_sing)", unInfl, cit));
                    // past
                    inflVar = unInfl + "ed";
                    past = new InflVar(eui, unInfl, type, "past", 
                        inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "past", unInfl, cit));
                    // past participle
                    inflVar = unInfl + "ed";
                    pastPart = new InflVar(eui, unInfl, type, "pastPart", 
                        inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "past_part", unInfl, cit));
                    // present participle
                    inflVar = unInfl + "ing";
                    presPart = new InflVar(eui, unInfl, type, "presPart", 
                        inflVar, cat, cit);
                    agrValues_.addElement(new Agreement(eui, inflVar, 
                        cat, "pres_part", unInfl, cit));
                }
                pres3s.SetUnique(IsUnique(inflValues_, pres3s));
                past.SetUnique(IsUnique(inflValues_, past));
                pastPart.SetUnique(IsUnique(inflValues_, pastPart));
                presPart.SetUnique(IsUnique(inflValues_, presPart));
                inflValues_.addElement(pres3s);
                inflValues_.addElement(past);
                inflValues_.addElement(pastPart);
                inflValues_.addElement(presPart);
            }
            else if(variant.startsWith("irreg"))
            {
                if(inflValues_.contains(infinitive) == false)
                {
                    inflValues_.addElement(infinitive);
                }
                if(agrValues_.contains(agrInf) == false)
                {
                    agrValues_.addElement(agrInf);
                }
                int index1 = variant.indexOf("|");
                int index2 = variant.indexOf("|", index1+1);
                int index3 = variant.indexOf("|", index2+1);
                int index4 = variant.indexOf("|", index3+1);
                int index5 = variant.indexOf("|", index4+1);
                int index6 = variant.indexOf("|", index5+1);
                String type = variant.substring(0, index1);
                String baseStr = variant.substring(index1+1, index2);
                String pres3sStr = variant.substring(index2+1, index3);
                String pastStr = variant.substring(index3+1, index4);
                String pastPartStr = variant.substring(index4+1, index5);
                String presPartStr = variant.substring(index5+1, index6);
                if(unInfl.equals(baseStr) == true)
                {
                    // present 3 ps
                    if(pres3sStr.length() > 0)
                    {
                        InflVar pres3s = new InflVar(eui, 
                            unInfl, type, "pres3s", pres3sStr, cat, cit);
                        pres3s.SetUnique(IsUnique(inflValues_, pres3s));
                        inflValues_.addElement(pres3s);
                        agrValues_.addElement(new Agreement(eui, pres3sStr, 
                            cat, "pres(thr_sing)", unInfl, cit));
                    }
                    // past
                    if(pastStr.length() > 0)
                    {
                        InflVar past = new InflVar(eui, 
                            unInfl, type, "past", pastStr, cat, cit);
                        past.SetUnique(IsUnique(inflValues_, past));
                        inflValues_.addElement(past);
                        agrValues_.addElement(new Agreement(eui, pastStr, 
                            cat, "past", unInfl, cit));
                    }
                    // past participle
                    if(pastPartStr.length() > 0)
                    {
                        InflVar pastPart = new InflVar(eui, 
                            unInfl, type, "pastPart", pastPartStr, cat, cit);
                        pastPart.SetUnique(IsUnique(inflValues_, pastPart));
                        inflValues_.addElement(pastPart);
                        agrValues_.addElement(new Agreement(eui, pastPartStr, 
                            cat, "past_part", unInfl, cit));
                    }
                    // presPart
                    if(presPartStr.length() > 0)
                    {
                        InflVar presPart = new InflVar(eui, 
                            unInfl, type, "presPart", presPartStr, cat, cit);
                        presPart.SetUnique(IsUnique(inflValues_, presPart));
                        inflValues_.addElement(presPart);
                        agrValues_.addElement(new Agreement(eui, presPartStr, 
                            cat, "pres_part", unInfl, cit));
                    }
                }
                // TBD: move to above
                /***
                if(pres3sStr.length() > 0)
                {
                    agrValues_.addElement(new Agreement(eui, pres3sStr, 
                        cat, "pres(thr_sing)", unInfl, cit));
                }
                if(pastStr.length() > 0)
                {
                    agrValues_.addElement(new Agreement(eui, pastStr, 
                        cat, "past", unInfl, cit));
                }
                if(pastPartStr.length() > 0)
                {
                    agrValues_.addElement(new Agreement(eui, pastPartStr, 
                        cat, "past_part", unInfl, cit));
                }
                if(presPartStr.length() > 0)
                {
                    agrValues_.addElement(new Agreement(eui, presPartStr, 
                        cat, "pres_part", unInfl, cit));
                }
                ***/
                
            }
            else
            {
                System.out.println("-- Warning: InflVarsAndAgreements.GetVerbInflVarsAndAgreements()");
            }
        }
    }
    private void GetNounInflVarsAndAgreements(String unInfl, String eui, 
        String cat, String cit)
    {
        // base & singular
        inflValues_.addElement(new InflVar(eui, unInfl, "base", 
            unInfl, cat, cit));
        InflVar singular = 
            new InflVar(eui, unInfl, "singular", unInfl, cat, cit);
        
        Agreement singAgr = new Agreement(eui, unInfl, cat, 
             "count(thr_sing)", unInfl, cit);
        // go through variants rules list
        Vector<String> variants 
            = lexRecord_.GetCatEntry().GetNounEntry().GetVariants();
        String inflVar = new String();
        for(int i = 0; i < variants.size(); i++)
        {
            String variant = variants.elementAt(i);
            if(variant.startsWith("reg"))
            {
                if(inflValues_.contains(singular) == false)
                {
                    inflValues_.addElement(singular);
                }
                if(agrValues_.contains(singAgr) == false)
                {
                    agrValues_.addElement(singAgr);
                }
                char lastChar = GetLastChar(unInfl);
                char last2Char = GetLast2Char(unInfl);
                String last2CharStr = new Character(last2Char).toString();
                InflVar plural = null;
                if((lastChar == 'y')
                && (consonants_.contains(last2CharStr)))
                {
                    // plural
                    inflVar = unInfl.substring(0, unInfl.length()-1) + "ies";
                    plural = new InflVar(eui, unInfl, "reg", "plural", 
                        inflVar, cat, cit);
                }
                else if((unInfl.endsWith("s"))
                || (unInfl.endsWith("z"))
                || (unInfl.endsWith("x"))
                || (unInfl.endsWith("ch"))
                || (unInfl.endsWith("sh")))
                {
                    // plural
                    inflVar = unInfl + "es";
                    plural = new InflVar(eui, unInfl, "reg", "plural", 
                        inflVar, cat, cit);
                }
                else
                {
                    // plural
                    inflVar = unInfl + "s";
                    plural = new InflVar(eui, unInfl, "reg", "plural", 
                        inflVar, cat, cit);
                }
                plural.SetUnique(IsUnique(inflValues_, plural));
                inflValues_.addElement(plural);
                agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                    "count(thr_plur)", unInfl, cit));
            }
            else if(variant.startsWith("glreg"))
            {
                if(inflValues_.contains(singular) == false)
                {
                    inflValues_.addElement(singular);
                }
                if(agrValues_.contains(singAgr) == false)
                {
                    agrValues_.addElement(singAgr);
                }
                inflVar = unInfl;
                if(unInfl.endsWith("us"))
                {
                    // plural: us -> i
                    inflVar = unInfl.substring(0, unInfl.length()-2) + "i";
                    inflValues_.addElement(new InflVar(eui, unInfl, "glreg", 
                        "plural", inflVar, cat, cit));
                }
                else if(unInfl.endsWith("ma"))
                {
                    // plural: ma -> mata
                    inflVar = unInfl + "ta";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        "glreg", "plural", inflVar, cat, cit));
                }
                else if(unInfl.endsWith("a"))
                {
                    // plural: a -> ae
                    inflVar = unInfl + "e";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        "glreg", "plural", inflVar, cat, cit));
                }
                else if(unInfl.endsWith("um"))
                {
                    // plural: um -> a
                    inflVar = unInfl.substring(0, unInfl.length()-2) + "a";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        "glreg", "plural", inflVar, cat, cit));
                }
                else if(unInfl.endsWith("on"))
                {
                    // plural: on -> a
                    inflVar = unInfl.substring(0, unInfl.length()-2) + "a";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        "glreg", "plural", inflVar, cat, cit));
                }
                else if(unInfl.endsWith("sis"))
                {
                    // plural: sis -> ses
                    inflVar = unInfl.substring(0, unInfl.length()-2) + "es";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        "glreg", "plural", inflVar, cat, cit));
                }
                else if(unInfl.endsWith("is"))
                {
                    // plural: is -> ides
                    inflVar = unInfl.substring(0, unInfl.length()-1) + "des";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        "glreg", "plural", inflVar, cat, cit));
                }
                else if(unInfl.endsWith("men"))
                {
                    // plural: men -> mina
                    inflVar = unInfl.substring(0, unInfl.length()-2) + "ina";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        "glreg", "plural", inflVar, cat, cit));
                }
                else if(unInfl.endsWith("ex"))
                {
                    // plural: ex -> ices
                    inflVar = unInfl.substring(0, unInfl.length()-2) + "ices";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        "glreg", "plural", inflVar, cat, cit));
                }
                else if(unInfl.endsWith("x"))
                {
                    // plural: x -> ces
                    inflVar = unInfl.substring(0, unInfl.length()-1) + "ces";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        "glreg", "plural", inflVar, cat, cit));
                }
                else    // base form is not one of above
                {
                    System.err.println("** Err@glreg violation: " + eui + "|"
                        + unInfl + "|" + cat);
                }
                agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                    "count(thr_plur)", unInfl, cit));
            }
            else if(variant.startsWith("metareg"))
            {
                if(inflValues_.contains(singular) == false)
                {
                    inflValues_.addElement(singular);
                }
                if(agrValues_.contains(singAgr) == false)
                {
                    agrValues_.addElement(singAgr);
                }
                inflVar = unInfl + "s";
                InflVar plural = new InflVar(eui, unInfl, "metareg", "plural", 
                    inflVar, cat, cit);
                plural.SetUnique(IsUnique(inflValues_, plural));
                inflValues_.addElement(plural);
                agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                    "count(thr_plur)", unInfl, cit));
                inflVar = unInfl + "'s";
                InflVar plural2 = new InflVar(eui, unInfl, "metareg", "plural", 
                    inflVar, cat, cit);
                plural2.SetUnique(IsUnique(inflValues_, plural2));
                inflValues_.addElement(plural2);
                agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                    "count(thr_plur)", unInfl, cit));
            }
            else if(variant.startsWith("irreg"))
            {
                if(inflValues_.contains(singular) == false)
                {
                    inflValues_.addElement(singular);
                }
                if(agrValues_.contains(singAgr) == false)
                {
                    agrValues_.addElement(singAgr);
                }
                int index1 = variant.indexOf("|");
                int index2 = variant.indexOf("|", index1+1);
                int index3 = variant.indexOf("|", index2+1);
                String type = variant.substring(0, index1);
                String baseStr = variant.substring(index1+1, index2);
                String pluralStr = variant.substring(index2+1, index3);
                // plural
                if(unInfl.equals(baseStr) == true)
                {
                    if(pluralStr.length() > 0)
                    {
                        InflVar plural = new InflVar(eui, 
                            unInfl, type, "plural", pluralStr, cat, cit);
                        plural.SetUnique(IsUnique(inflValues_, plural));
                        inflValues_.addElement(plural);
                        agrValues_.addElement(new Agreement(eui, pluralStr, cat,
                            "count(thr_plur)", unInfl, cit));
                    }
                }
                // TBD: should move up
                /**
                agrValues_.addElement(new Agreement(eui, pluralStr, cat,
                    "count(thr_plur)", unInfl, cit));
                **/
            }
            else if(variant.startsWith("sing"))
            {
                if(inflValues_.contains(singular) == false)
                {
                    inflValues_.addElement(singular);
                }
                if(agrValues_.contains(singAgr) == false)
                {
                    agrValues_.addElement(singAgr);
                }
            }
            else if(variant.startsWith("plur"))
            {
                InflVar plural = new InflVar(eui, unInfl, 
                    "plur", "plural", unInfl, cat, cit);
                //check if unique
                plural.SetUnique(IsUnique(inflValues_, plural));
                inflValues_.addElement(plural);
                agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                    "count(thr_plur)", unInfl, cit));
            }
            else if(variant.startsWith("inv"))
            {
                if(inflValues_.contains(singular) == false)
                {
                    inflValues_.addElement(singular);
                }
                if(agrValues_.contains(singAgr) == false)
                {
                    agrValues_.addElement(singAgr);
                }
                InflVar plural = new InflVar(eui, unInfl, "inv",
                    "plural", unInfl, cat, cit);
                plural.SetUnique(IsUnique(inflValues_, plural));
                inflValues_.addElement(plural);
                agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                    "count(thr_plur)", unInfl, cit));
            }
            else if(variant.startsWith("uncount"))    // no plural form
            {
                if(inflValues_.contains(singular) == false)
                {
                    inflValues_.addElement(singular);
                }
                agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                    "uncount(thr_sing)", unInfl, cit));
            }
            else if(variant.startsWith("groupuncount"))
            {
                if(inflValues_.contains(singular) == false)
                {
                    inflValues_.addElement(singular);
                }
                agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                    "uncount(thr_sing)", unInfl, cit));
                InflVar plural = new InflVar(eui, unInfl, 
                    "groupuncount", "plural", unInfl, cat, cit);
                plural.SetUnique(IsUnique(inflValues_, plural));
                inflValues_.addElement(plural);
                agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                    "uncount(thr_plur)", unInfl, cit));
            }
            else if(variant.startsWith("group("))
            {
                if(inflValues_.contains(singular) == false)
                {
                    inflValues_.addElement(singular);
                }
                if(agrValues_.contains(singAgr) == false)
                {
                    agrValues_.addElement(singAgr);
                }
                // group means itself can be used as plural form
                InflVar plur = new InflVar(eui, unInfl,
                    variant, "plural", unInfl, cat, cit);
                plur.SetUnique(IsUnique(inflValues_, plur));
                // don't duplicate if it's irreg with spelling vars
                if((IsUnique(inflValues_, plur) == true)
                && (inflValues_.contains(plur) == false))
                {
                    inflValues_.addElement(plur);
                    agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                        "count(thr_plur)", unInfl, cit));
                }
                // get the argument in the group(argument)
                String argument = variant.substring(6, variant.length()-1);
                if(argument.equals("reg"))
                {
                    char lastChar = GetLastChar(unInfl);
                    char last2Char = GetLast2Char(unInfl);
                    String last2CharStr = new Character(last2Char).toString();
                    InflVar plural = null;
                    if((lastChar == 'y')
                    && (consonants_.contains(last2CharStr)))
                    {
                        // plural
                        inflVar = unInfl.substring(0, unInfl.length()-1) +"ies";
                        plural = new InflVar(eui, unInfl, variant, "plural", 
                            inflVar, cat, cit);
                    }
                    else if((unInfl.endsWith("s"))
                    || (unInfl.endsWith("z"))
                    || (unInfl.endsWith("x"))
                    || (unInfl.endsWith("ch"))
                    || (unInfl.endsWith("sh")))
                    {
                        // plural
                        inflVar = unInfl + "es";
                        plural = new InflVar(eui, unInfl, variant, "plural", 
                            inflVar, cat, cit);
                    }
                    else
                    {
                        // plural
                        inflVar = unInfl + "s";
                        plural = new InflVar(eui, unInfl, variant, "plural", 
                            inflVar, cat, cit);
                    }
                    plural.SetUnique(IsUnique(inflValues_, plural));
                    inflValues_.addElement(plural);
                    agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                        "count(thr_plur)", unInfl, cit));
                }
                else if(argument.equals("glreg"))
                {
                    if(unInfl.endsWith("us"))
                    {
                        // plural: us -> i
                        inflVar = unInfl.substring(0, unInfl.length()-2) + "i";
                        inflValues_.addElement(new InflVar(eui, unInfl, 
                            variant, "plural", inflVar, cat, cit));
                    }
                    else if(unInfl.endsWith("ma"))
                    {
                        // plural: ma -> mata
                        inflVar = unInfl + "ta";
                        inflValues_.addElement(new InflVar(eui, unInfl,
                            variant, "plural", inflVar, cat, cit));
                    }
                    else if(unInfl.endsWith("a"))
                    {
                        // plural: a -> ae
                        inflVar = unInfl + "e";
                        inflValues_.addElement(new InflVar(eui, unInfl,
                            variant, "plural", inflVar, cat, cit));
                    }
                    else if(unInfl.endsWith("um"))
                    {
                        // plural: um -> a
                        inflVar = unInfl.substring(0, unInfl.length()-2) + "a";
                        inflValues_.addElement(new InflVar(eui, unInfl, 
                            variant, "plural", inflVar, cat, cit));
                    }
                    else if(unInfl.endsWith("on"))
                    {
                        // plural: on -> a
                        inflVar = unInfl.substring(0, unInfl.length()-2) + "a";
                        inflValues_.addElement(new InflVar(eui, unInfl,
                            variant, "plural", inflVar, cat, cit));
                    }
                    else if(unInfl.endsWith("sis"))
                    {
                        // plural: sis -> ses
                        inflVar = unInfl.substring(0, unInfl.length()-2) + "es";
                        inflValues_.addElement(new InflVar(eui, unInfl, 
                            variant, "plural", inflVar, cat, cit));
                    }
                    else if(unInfl.endsWith("is"))
                    {
                        // plural: is -> ides
                        inflVar = unInfl.substring(0, unInfl.length()-1) +"des";
                        inflValues_.addElement(new InflVar(eui, unInfl, 
                            variant, "plural", inflVar, cat, cit));
                    }
                    else if(unInfl.endsWith("men"))
                    {
                        // plural: men -> mina
                        inflVar = unInfl.substring(0, unInfl.length()-2) +"ina";
                        inflValues_.addElement(new InflVar(eui, unInfl,
                            variant, "plural", inflVar, cat, cit));
                    }
                    else if(unInfl.endsWith("ex"))
                    {
                        // plural: ex -> ices
                        inflVar = unInfl.substring(0, unInfl.length()-2)+"ices";
                        inflValues_.addElement(new InflVar(eui, unInfl,
                            variant, "plural", inflVar, cat, cit));
                    }
                    else if(unInfl.endsWith("x"))
                    {
                        // plural: x -> ces
                        inflVar = unInfl.substring(0, unInfl.length()-1) +"ces";
                        inflValues_.addElement(new InflVar(eui, unInfl, 
                            variant, "plural", inflVar, cat, cit));
                    }
                    else
                    {
                        System.err.println("** Err@glreg violation: " + eui 
                            + "|" + unInfl + "|" + cat);
                    }
                    agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                        "count(thr_plur)", unInfl, cit));
                }
                else if(argument.equals("metareg"))
                {
                    inflVar = unInfl + "s";
                    InflVar plural = new InflVar(eui, unInfl,
                        variant, "plural", inflVar, cat, cit);
                    plural.SetUnique(IsUnique(inflValues_, plural));
                    inflValues_.addElement(plural);
                    agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                        "count(thr_plur)", unInfl, cit));
                    inflVar = unInfl + "'s";
                    InflVar plural2 = new InflVar(eui, unInfl, 
                        variant, "plural", inflVar, cat, cit);
                    plural2.SetUnique(IsUnique(inflValues_, plural2));
                    inflValues_.addElement(plural2);
                    agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                        "count(thr_plur)", unInfl, cit));
                }
                else if(argument.startsWith("irreg"))
                {
                    int index1 = argument.indexOf("|");
                    int index2 = argument.indexOf("|", index1+1);
                    int index3 = argument.indexOf("|", index2+1);
                    String tagStr = argument.substring(0, index1);
                    String baseStr = argument.substring(index1+1, index2);
                    String pluralStr = argument.substring(index2+1, index3);
                    // plural
                    if(unInfl.equals(baseStr) == true)
                    {
                        if(pluralStr.length() > 0)
                        {
                            InflVar plural = new InflVar(eui, 
                                unInfl, variant, "plural", pluralStr, cat, cit);
                            plural.SetUnique(IsUnique(inflValues_, plural));
                            inflValues_.addElement(plural);
                            agrValues_.addElement(new Agreement(eui, pluralStr,
                                cat, "count(thr_plur)", unInfl, cit));
                        }
                    }
                }
                else    // sing
                {
                    // nothing to be do for this
                }
            }
            else
            {
                System.out.println("-- Warning: InflVarsAndAgreements.GetNounInflVarsAndAgreements()" + variant);
            }
        }
    }
    private void GetAdjInflVarsAndAgreements(String unInfl, String eui, 
        String cat, String cit)
    {
        // base & positive
        inflValues_.addElement(new InflVar(eui, unInfl, "base", 
            unInfl, cat, cit));
        InflVar positive = 
            new InflVar(eui, unInfl, "positive", unInfl, cat, cit);
        inflValues_.addElement(positive);
        // go through variants list
        Vector<String> variants 
            = lexRecord_.GetCatEntry().GetAdjEntry().GetVariants();
        String inflVar = new String();    
        for(int i = 0; i < variants.size(); i++)
        {
            String variant = variants.elementAt(i);
            if(variant.startsWith("irreg"))
            {
                int index1 = variant.indexOf("|");
                int index2 = variant.indexOf("|", index1+1);
                int index3 = variant.indexOf("|", index2+1);
                int index4 = variant.indexOf("|", index3+1);
                String type = variant.substring(0, index1);
                String baseStr = variant.substring(index1+1, index2);
                String compStr = variant.substring(index2+1, index3);
                String superStr = variant.substring(index3+1, index4);
                agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                    "positive", unInfl, cit));
                // comparative
                if(unInfl.equals(baseStr) == true)
                {
                    if(compStr.length() > 0)
                    {
                        InflVar comparative = new InflVar(eui, 
                            unInfl, type, "comparative", compStr, cat, cit);
                        comparative.SetUnique(IsUnique(inflValues_, 
                            comparative));
                        inflValues_.addElement(comparative);
                        agrValues_.addElement(new Agreement(eui, compStr, cat, 
                            "comparative", unInfl, cit));
                    }
                    // superlative
                    if(superStr.length() > 0)
                    {
                        InflVar superlative = new InflVar(eui, 
                            unInfl, type, "superlative", superStr, cat, cit);
                        superlative.SetUnique(IsUnique(inflValues_, 
                            superlative));
                        inflValues_.addElement(superlative);
                        agrValues_.addElement(new Agreement(eui, superStr, cat, 
                            "superlative", unInfl, cit));
                    }
                }
            }
            else if(variant.startsWith("regd"))
            {
                String type = "regd";
                char lastChar = GetLastChar(unInfl);
                char last2Char = GetLast2Char(unInfl);
                String lastCharStr = new Character(lastChar).toString();
                String last2CharStr = new Character(last2Char).toString();
                // check if the base form ends with VC
                if((consonants_.contains(lastCharStr) == false)
                || (vowels_.contains(last2CharStr) == false))
                {
                    System.err.println("** Err@regd violation: " + eui + "|"
                        + unInfl + "|" + cat);
                }
                agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                    "positive", unInfl, cit));
                // comparative
                inflVar = unInfl + lastChar +  "er";
                inflValues_.addElement(new InflVar(eui, unInfl, type, 
                    "comparative", inflVar, cat, cit));
                agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                    "comparative", unInfl, cit));
                // superlative
                inflVar = unInfl + lastChar + "est";
                inflValues_.addElement(new InflVar(eui, unInfl, type, 
                    "superlative", inflVar, cat, cit));
                agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                    "superlative", unInfl, cit));
            }
            else if(variant.startsWith("reg"))
            {
                String type = "reg";
                char lastChar = GetLastChar(unInfl);
                char last2Char = GetLast2Char(unInfl);
                String last2CharStr = new Character(last2Char).toString();
                agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                    "positive", unInfl, cit));
                if((lastChar == 'y')
                && (consonants_.contains(last2CharStr)))
                {
                    // comparative
                    inflVar = unInfl.substring(0, unInfl.length()-1) + "ier";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        type, "comparative", inflVar, cat, cit));
                    agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                        "comparative", unInfl, cit));
                    // superlative
                    inflVar = unInfl.substring(0, unInfl.length()-1) + "iest";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        type, "superlative", inflVar, cat, cit));
                    agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                        "superlative", unInfl, cit));
                }
                else if(lastChar == 'e')
                // && (consonants_.contains(last2CharStr))) To be deleted
                {
                    // comparative
                    inflVar = unInfl + "r";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        type, "comparative", inflVar, cat, cit));
                    agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                        "comparative", unInfl, cit));
                    // superlative
                    inflVar = unInfl + "st";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        type, "superlative", inflVar, cat, cit));
                    agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                        "superlative", unInfl, cit));
                }
                else // others ((lastChar != 'e') && (lastChar != 'y')) 
                {
                    // comparative
                    inflVar = unInfl + "er"; 
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        type, "comparative", inflVar, cat, cit));
                    agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                        "comparative", unInfl, cit));
                    // superlative
                    inflVar = unInfl + "est";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        type, "superlative", inflVar, cat, cit));
                    agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                        "superlative", unInfl, cit));
                }
            }
            else if(variant.startsWith("inv;periph")) // inv;periph
            {
                agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                    "positive;periph", unInfl, cit));
            }
            else    // inv
            {
                agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                    "positive", unInfl, cit));
            }
        }
    }
    private void GetAdvInflVarsAndAgreements(String unInfl, String eui, 
        String cat, String cit)
    {
        // base
        inflValues_.addElement(new InflVar(eui, unInfl, "base", 
            unInfl, cat, cit));
        InflVar positive = 
            new InflVar(eui, unInfl, "positive", unInfl, cat, cit);
        inflValues_.addElement(positive);
        // go through variants list
        Vector<String> variants 
            = lexRecord_.GetCatEntry().GetAdvEntry().GetVariants();
        String inflVar = new String();    
        for(int i = 0; i < variants.size(); i++)
        {
            String variant = variants.elementAt(i);
            if(variant.startsWith("irreg"))
            {
                int index1 = variant.indexOf("|");
                int index2 = variant.indexOf("|", index1+1);
                int index3 = variant.indexOf("|", index2+1);
                int index4 = variant.indexOf("|", index3+1);
                String type = variant.substring(0, index1);
                String baseStr = variant.substring(index1+1, index2);
                String comparative = variant.substring(index2+1, index3);
                String superlative = variant.substring(index3+1, index4);
                agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                    "positive", unInfl, cit));
                if(unInfl.equals(baseStr) == true)
                {
                    // comparative
                    if(comparative.length() > 0)
                    {
                        inflValues_.addElement(new InflVar(eui, unInfl, 
                            type, "comparative", comparative, cat, cit));
                        /**
                        agrValues_.addElement(new Agreement(eui, comparative, 
                            cat, "comparative", unInfl, cit));
                        **/
                    }
                    // superlative
                    if(superlative.length() > 0)
                    {
                        inflValues_.addElement(new InflVar(eui, unInfl, 
                            type, "superlative", superlative, cat, cit));
                        /**
                        agrValues_.addElement(new Agreement(eui, superlative, 
                            cat, "superlative", unInfl, cit));
                        **/
                    }
                }
                if(comparative.length() > 0)
                {
                    agrValues_.addElement(new Agreement(eui, comparative, 
                        cat, "comparative", unInfl, cit));
                }
                if(superlative.length() > 0)
                {
                    agrValues_.addElement(new Agreement(eui, superlative, 
                        cat, "superlative", unInfl, cit));
                }
            }
            else if(variant.startsWith("reg"))
            {
                String type = "reg";
                char lastChar = GetLastChar(unInfl);
                char last2Char = GetLast2Char(unInfl);
                String last2CharStr = new Character(last2Char).toString();
                agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                    "positive", unInfl, cit));
                if((lastChar == 'y')
                && (consonants_.contains(last2CharStr)))
                {
                    // comparative
                    inflVar = unInfl.substring(0, unInfl.length()-1) + "ier";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        type, "comparative", inflVar, cat, cit));
                    agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                        "comparative", unInfl, cit));
                    // superlative
                    inflVar = unInfl.substring(0, unInfl.length()-1) + "iest";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        type, "superlative", inflVar, cat, cit));
                    agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                        "superlative", unInfl, cit));
                }
                else if(lastChar == 'e')
                //&& (consonants_.contains(last2CharStr)))  to be deleted
                {
                    // comparative
                    inflVar = unInfl + "r";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        type, "comparative", inflVar, cat, cit));
                    agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                        "comparative", unInfl, cit));
                    // superlative
                    inflVar = unInfl + "st";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        type, "superlative", inflVar, cat, cit));
                    agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                        "superlative", unInfl, cit));
                }
                else // others ((lastChar != 'e') && (lastChar != 'y')) 
                {
                    // comparative
                    inflVar = unInfl + "er";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        type, "comparative", inflVar, cat, cit));
                    agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                        "comparative", unInfl, cit));
                    // superlative
                    inflVar = unInfl + "est";
                    inflValues_.addElement(new InflVar(eui, unInfl, 
                        type, "superlative", inflVar, cat, cit));
                    agrValues_.addElement(new Agreement(eui, inflVar, cat, 
                        "superlative", unInfl, cit));
                }
            }
            else if(variant.startsWith("inv;periph"))        // inv;periph
            {
                agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                    "positive;periph", unInfl, cit));
            }
            else    // inv, 
            {
                agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                    "positive", unInfl, cit));
            }
        }
    }
    public static char GetLastChar(String in)
    {
        int length = in.length();
        char out = in.toLowerCase().charAt(length-1);
        return out;
    }
    public static char GetLast2Char(String in)
    {
        char out = ' ';    // an empty character
        int length = in.length();
        if(length >= 2)
        {
            out = in.toLowerCase().charAt(length-2);
        }
        return out;
    }
    private void GetAuxInflVarsAndAgreements(String unInfl, String eui, 
        String cat, String cit)
    {
        // base
        inflValues_.addElement(new InflVar(eui, unInfl, "base", 
            unInfl, cat, cit));
        // go through variants list
        Vector<String> variants 
            = lexRecord_.GetCatEntry().GetAuxEntry().GetVariant();
        for(int i = 0; i < variants.size(); i++)
        {
            String variant = variants.elementAt(i);
            int index = variant.indexOf(";");
            String value = variant.substring(0, index);
            int index1 = variant.indexOf("(", index);
            int index2 = variant.indexOf(":", index);
            String inflStr = variant.substring(index+1);
            String infl = MapToInflection(inflStr);
            inflValues_.addElement(new InflVar(eui, unInfl, "reg", infl,
                value, cat, cit));
            agrValues_.addElement(new Agreement(eui, value, cat, 
                inflStr, unInfl, cit));
        }
    }
    private void GetModalInflVarsAndAgreements(String unInfl, String eui, 
        String cat, String cit)
    {
        // base & present
        inflValues_.addElement(new InflVar(eui, unInfl, "base", unInfl,
            cat, cit));
        inflValues_.addElement(new InflVar(eui, unInfl, "pres", unInfl,
            cat, cit));
        agrValues_.addElement(new Agreement(eui, unInfl, cat, 
            "pres", unInfl, cit));
        // go through variants list
        Vector<String> variants 
            = lexRecord_.GetCatEntry().GetModalEntry().GetVariant();
        for(int i = 0; i < variants.size(); i++)
        {
            String variant = variants.elementAt(i);
            int index = variant.indexOf(";");
            String value = variant.substring(0, index);
            int index1 = variant.indexOf("(", index);
            int index2 = variant.indexOf(":", index);
            String inflStr = variant.substring(index+1);
            String infl = MapToInflection(inflStr);
            inflValues_.addElement(new InflVar(eui, unInfl, "reg", infl,
                value, cat, cit));
            agrValues_.addElement(new Agreement(eui, value, cat, 
                inflStr, unInfl, cit));
        }
    }
    private String MapToInflection(String in)
    {
        // Tokenize input
        String delim = " (,):";
        StringTokenizer buf = new StringTokenizer(in, delim);
        String out = new String();
        while(buf.hasMoreTokens() == true)
        {
            String curToken = buf.nextToken();
            if(curToken.equals("past"))
            {
                out += "past";
            }
            else if(curToken.equals("pres"))
            {
                out += "pres";
            }
            else if(curToken.equals("past_part"))
            {
                out += "pastPart";
            }
            else if(curToken.equals("pres_part"))
            {
                out += "presPart";
            }
            else if(curToken.equals("infinitive"))
            {
                out += "infinitive";
            }
            else if(curToken.equals("negative"))
            {
                out += "Neg";
            }
            else if(curToken.equals("fst_sing"))
            {
                out += "1s";
            }
            else if(curToken.equals("fst_plur"))
            {
                out += "1p";
            }
            else if(curToken.equals("second"))
            {
                out += "2";
            }
            else if(curToken.equals("sec_sing"))
            {
                out += "2s";
            }
            else if(curToken.equals("sec_plur"))
            {
                out += "2p";
            }
            else if(curToken.equals("third"))
            {
                out += "3";
            }
            else if(curToken.equals("thr_sing"))
            {
                out += "3s";
            }
            else if(curToken.equals("thr_plur"))
            {
                out += "3p";
            }
        }
        // replace 1s1p to 1
        int index = out.indexOf("1s1p");
        if(index > 0)
        {
            out = out.substring(0, index+1) + out.substring(index+4);
        }
        if(inflections_.contains(out) == false)
        {
            System.out.println("** Error: inflection '" + out + "' is illegal");
            out = "Error";
        }
        return out;
    }
    private void GetPronInflVarsAndAgreements(String unInfl, String eui, 
        String cat, String cit)
    {
        inflValues_.addElement(new InflVar(eui, unInfl, "base", 
            unInfl, cat, cit));
        // arguments
        // go through variants list
        Vector<String> variants 
            = lexRecord_.GetCatEntry().GetPronEntry().GetVariants();
        String agreement = new String();    
        for(int i = 0; i < variants.size(); i++)
        {
            agreement = variants.elementAt(i);
            agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                agreement, unInfl, cit));
        }
    }
    private void GetDetInflVarsAndAgreements(String unInfl, String eui, 
        String cat, String cit)
    {
        inflValues_.addElement(new InflVar(eui, unInfl, "base", 
            unInfl, cat, cit));
        // arguments
        String agreement = lexRecord_.GetCatEntry().GetDetEntry().GetVariants();
        if(agreement.equals("singuncount"))
        {
            agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                "sing", unInfl, cit));
            agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                "uncount", unInfl, cit));
        }
        else if(agreement.equals("pluruncount"))
        {
            agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                "plur", unInfl, cit));
            agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                "uncount", unInfl, cit));
        }
        else
        {
            agrValues_.addElement(new Agreement(eui, unInfl, cat, 
                agreement, unInfl, cit));
        }
    }
    private void GetPrepInflVarsAndAgreements(String unInfl, String eui, 
        String cat, String cit)
    {
        inflValues_.addElement(new InflVar(eui, unInfl, "base", 
            unInfl, cat, cit));
        // argument
        agrValues_.addElement(new Agreement(eui, unInfl, cat, 
            new String(), unInfl, cit));
    }
    private void GetComplInflVarsAndAgreements(String unInfl, String eui, 
        String cat, String cit)
    {
        inflValues_.addElement(new InflVar(eui, unInfl, "base", 
            unInfl, cat, cit));
        // argument
        agrValues_.addElement(new Agreement(eui, unInfl, cat, 
            new String(), unInfl, cit));
    }
    private void GetConjInflVarsAndAgreements(String unInfl, String eui, 
        String cat, String cit)
    {
        inflValues_.addElement(new InflVar(eui, unInfl, "base", 
            unInfl, cat, cit));
        // argument
        agrValues_.addElement(new Agreement(eui, unInfl, cat, 
            new String(), unInfl, cit));
    }
    // check if inflVar is the first unique in inflVars
    // includes, eui, cat, infl, var, cit, unInfl
    private boolean IsUnique(Vector<InflVar> inflVars, InflVar inflVar)
    {
        boolean isUnique = true;
        for(int i = 0; i < inflVars.size(); i++)
        {
            InflVar cur = inflVars.elementAt(i);
            if((inflVar.GetEui() == null) || (cur.GetEui() == null))
            {
                if((inflVar.GetVar().equals(cur.GetVar()))
                && (inflVar.GetInflection().equals(cur.GetInflection()))
                && (inflVar.GetCat().equals(cur.GetCat()))
                && (inflVar.GetCit().equals(cur.GetCit()))
                && (inflVar.GetUnInfl().equals(cur.GetUnInfl())))
                {
                    isUnique = false;
                    break; 
                }
            }
            else
            {
                if((inflVar.GetVar().equals(cur.GetVar()))
                && (inflVar.GetInflection().equals(cur.GetInflection()))
                && (inflVar.GetCat().equals(cur.GetCat()))
                && (inflVar.GetCit().equals(cur.GetCit()))
                && (inflVar.GetEui().equals(cur.GetEui()))
                && (inflVar.GetUnInfl().equals(cur.GetUnInfl())))
                {
                    isUnique = false;
                    break; 
                }
            }
        }
        return isUnique;
    }
    // data member
    private LexRecord lexRecord_ = null;
    private Vector<InflVar> inflValues_ = new Vector<InflVar>();
    private Vector<Agreement> agrValues_ = new Vector<Agreement>();
    private static HashSet<String> inflections_ = new HashSet<String>();
    public static HashSet<String> consonants_ = new HashSet<String>();
    public static HashSet<String> vowels_ = new HashSet<String>();
    private static HashSet<String> eioySets_ = new HashSet<String>();
    static
    {
        inflections_.add("base");
        inflections_.add("comparative");
        inflections_.add("superlative");
        inflections_.add("plural");
        inflections_.add("presPart");
        inflections_.add("past");
        inflections_.add("pastPart");
        inflections_.add("pres3s");
        inflections_.add("positive");
        inflections_.add("singular");
        inflections_.add("infinitive");
        inflections_.add("pres123p");
        inflections_.add("pastNeg");
        inflections_.add("pres123pNeg");
        inflections_.add("pres1s");
        inflections_.add("past1p23pNeg");
        inflections_.add("past1p23p");
        inflections_.add("past1s3sNeg");
        inflections_.add("pres1p23p");
        inflections_.add("pres1p23pNeg");
        inflections_.add("past1s3s");
        inflections_.add("pres");
        inflections_.add("pres3sNeg");
        inflections_.add("presNeg");
        vowels_.add("a");
        vowels_.add("e");
        vowels_.add("i");
        vowels_.add("o");
        vowels_.add("u");
        consonants_.add("b");
        consonants_.add("c");
        consonants_.add("d");
        consonants_.add("f");
        consonants_.add("g");
        consonants_.add("h");
        consonants_.add("j");
        consonants_.add("k");
        consonants_.add("l");
        consonants_.add("m");
        consonants_.add("n");
        consonants_.add("p");
        consonants_.add("q");
        consonants_.add("r");
        consonants_.add("s");
        consonants_.add("t");
        consonants_.add("v");
        consonants_.add("w");
        consonants_.add("x");
        consonants_.add("y");
        consonants_.add("z");
        eioySets_.add("e");
        eioySets_.add("i");
        eioySets_.add("o");
        eioySets_.add("y");
    }
}
