package gov.nih.nlm.nls.lexCheck.Lib;
import java.util.*;
/*****************************************************************************
* This class provides a Java object for noun entry.
*
* <p><b>History:</b>
*
* @author NLM Lexical System Development Team
*
* @version    V-2019
****************************************************************************/
public class NounEntry
{
    /**
    *    Creates a default noun entry object
    */
    public NounEntry()
    {
        variants_ = new Vector<String>();
        compl_ = new Vector<String>();
        nominalization_ = new Vector<String>();
        tradeName_ = new Vector<String>();
        tradeMark_ = false;
        proper_ = false;
    }
    // public method
    /**
    * Get the collection of modifications of a noun entry
    *
    * @return  a Vector with modification elements.
    */
    public Vector<String> GetVariants()
    {
        return variants_;
    }
    /**
    * Get the collection of complement of a noun entry
    *
    * @return  a Vector with complement elements.
    */
    public Vector<String> GetCompl()
    {
        return compl_;
    }
    /**
    * Get the collection of nominalization of a noun entry
    *
    * @return  a Vector with nominalization elements.
    */
    public Vector<String> GetNominalization()
    {
        return nominalization_;
    }
    /**
    * Get the collection of trade name of a noun entry
    *
    * @return  a vector with trade name elements
    */
    public Vector<String> GetTradeName()
    {
        return tradeName_;
    }
    /**
    * Get the flag of trade mark of a noun entry
    *
    * @return  the boolean flag of a trade mark
    */
    public boolean IsTradeMark()
    {
        return tradeMark_;
    }
    /**
    * Get the flag of proper of a noun entry
    *
    * @return  the boolean flag of proper noun
    */
    public boolean IsProper()
    {
        return proper_;
    }
    /**
    * Add an inflectional variant to the collection of inflectional variants 
    * of a noun entry.
    *
    * @param  variant an inflectional variant to be added.
    */
    public void AddVariant(String variant)
    {
        variants_.add(variant);
    }
    /**
    * Set the collection of inflectional variants of a noun entry
    *
    * @param  variants   a Vector of inflectional variants to be set.
    */
    public void SetVariants(Vector<String> variants)
    {
        variants_ = variants;
    }
    /**
    * Add a complement to the collection of complements of a noun entry.
    *
    * @param  compl   a complement to be added.
    */
    public void AddCompl(String compl)
    {
        compl_.add(compl);
    }
    /**
    * Set the collection of complements of a noun entry
    *
    * @param  compl   a Vector of complements to be set.
    */
    public void SetCompl(Vector<String> compl)
    {
        compl_ = compl;
    }
    /**
    * Add a nominalization to the collection of nominalizations 
    * of a noun entry.
    *
    * @param  nominalization   a nominalization to be added.
    */
    public void AddNominalization(String nominalization)
    {
        nominalization_.add(nominalization);
    }
    /**
    * Set the nominalization of a noun entry
    *
    * @param  nominalization    a Vector of inflectional variants to be set.
    */
    public void SetNominalization(Vector<String> nominalization)
    {
        nominalization_ = nominalization;
    }
    /**
    * Add a tradeName to the collection of tradeNames 
    * of a noun entry.
    *
    * @param  tradeName   a tradeName to be added.
    */
    public void AddTradeName(String tradeName)
    {
        tradeName_.add(tradeName);
    }
    /**
    * Set the trade name of a noun entry
    *
    * @param  tradeName    a Vector of trade name to be set.
    */
    public void SetTradeName(Vector<String> tradeName)
    {
        tradeName_ = tradeName;
    }
    /**
    * Set the flag of trade mark of a noun entry
    *
    * @param  tradeMark    the flag of trade mark.
    */
    public void SetTradeMark(boolean tradeMark)
    {
        tradeMark_ = tradeMark;
    }
    /**
    * Set the flag of proper of a noun entry
    *
    * @param  proper    the flag of proper noun.
    */
    public void SetProper(boolean proper)
    {
        proper_ = proper;
    }
    /**
    * Get the text format of the noun entry
    *
    * @return  text format of the noun entry
    */
    public String GetText()
    {
        String text = new String();
        text = TextLib.AddToText(text, "variants=", variants_, 1);
        text = TextLib.AddToText(text, "compl=", compl_, 1);
        text = TextLib.AddToText(text, "nominalization_of=", nominalization_, 
            1);
        text = TextLib.AddToText(text, "proper", proper_, 1);
        text = TextLib.AddToText(text, "trademark=", tradeName_, 1);
        text = TextLib.AddToText(text, "trademark", tradeMark_, 1);
        return text;
    }
    /**
    * Get the xml format of the noun entry
    *
    * @return  xml format of the noun entry
    */
    public String GetXml()
    {
        boolean convertFlag = true;
        String xml = new String();
        xml = XmlLib.AddToXml(xml, "<nounEntry>", 2); 
        xml = XmlLib.AddToXml(xml, "<variants>", "</variants>", variants_, 3, 
            convertFlag);
        xml = XmlLib.AddToXml(xml, "<compl>", "</compl>", compl_, 3, 
            convertFlag);
        xml = XmlLib.AddToXml(xml, "<nominalization>", "</nominalization>", 
            nominalization_, 3, convertFlag);
        xml = XmlLib.AddToXml(xml, "<tradeName>", "</tradeName>", tradeName_, 
            3, convertFlag);
        xml = XmlLib.AddToXml(xml, "<trademark/>", tradeMark_, 3);
        xml = XmlLib.AddToXml(xml, "<proper/>", proper_, 3);
        xml = XmlLib.AddToXml(xml, "</nounEntry>", 2); 
        return xml;
    }
    // data member
    private Vector<String> variants_ = new Vector<String>();
    private Vector<String> compl_ = new Vector<String>();
    private Vector<String> nominalization_ = new Vector<String>();
    private Vector<String> tradeName_ = new Vector<String>();
    private boolean tradeMark_ = false;
    private boolean proper_ = false;
}
