package gov.nih.nlm.nls.lexCheck.Tools;
import java.io.*;
import java.util.*;
import java.text.*;
import java.nio.file.*;
import java.nio.charset.*;
/*****************************************************************************
* This class is to analyze new EUI file (from acronym or abbreviation) from 
* the tagged files.
* <ul>
* <li>Inputs: 
*   <ul>
*   <li>2.13.newEui.acr.tagged.txt or
*   <li>2.13.newEui.abb.tagged.txt
*   </ul>
* <li>Outputs:
*   <ul>
*   <li>input.y: yes tag, valid MW
*   <li>input.n: no tag, invalid MW
*   <li>input.i: i tag, valid inflections
*   <li>input.o: others, not above two (should be 0)
*   <li>input.out: invalid MW, out format: word|cat
*   </ul>
* </ul>
*
* <p><b>History:</b>
*
* @author NLM Lexical Systems Group
*
* @version    V-2019
*****************************************************************************/
public class AnalyzeNewEuiFile
{
    // private constructor
    private AnalyzeNewEuiFile()
    {
    }
    // test driver
	/**
	* Analyze new EUI file.
	*
	* @param  args  input arguments
	*/
    public static void main(String[] args)
    {
        String topDir =
            "/nfsvol/lex/Lu/Development/Lexicon/data/2017/data/Tags/";
        String inFile = topDir + "2.13.newEui.acr.tagged.txt";
        if(args.length == 1)
        {
            inFile = args[0];
        }
        else if(args.length >= 0)
        {
            System.err.println("Usage: java AnalyzeNewEuiFile <inFile>");
            System.exit(0);
        }
        // generate PmidTiAbFiles from medlineFile${YEAR}.txt
        AnalyzeNewEuiFromFile(inFile);
    }
    // private methods
    private static void AnalyzeNewEuiFromFile(String inFile)
    {
        System.out.println("===== AnalyzeNewEuiFromFile =====");
        int lineNo = 0; // line no
        String line = null;
        String yFile = inFile + ".y";    // Y tag (base)
        String nFile = inFile + ".n";    // N tag (not valid LexWord)
        String iFile = inFile + ".i";    // I tag (inflection)
        String oFile = inFile + ".o";    // Others
        String notBaseFile = inFile + ".notBase";  // format: word|cat, not Base
        String notLmwFile = inFile + ".notLmw";    // format: word|cat, not LMW
        try
        {
            BufferedReader reader = Files.newBufferedReader(
                Paths.get(inFile), Charset.forName("UTF-8"));
            BufferedWriter yWriter = Files.newBufferedWriter(
                Paths.get(yFile), Charset.forName("UTF-8"));
            BufferedWriter iWriter = Files.newBufferedWriter(
                Paths.get(iFile), Charset.forName("UTF-8"));
            BufferedWriter nWriter = Files.newBufferedWriter(
                Paths.get(nFile), Charset.forName("UTF-8"));
            BufferedWriter oWriter = Files.newBufferedWriter(
                Paths.get(oFile), Charset.forName("UTF-8"));
            BufferedWriter notBaseWriter = Files.newBufferedWriter(
                Paths.get(notBaseFile), Charset.forName("UTF-8"));
            BufferedWriter notLmwWriter = Files.newBufferedWriter(
                Paths.get(notLmwFile), Charset.forName("UTF-8"));
            // print out the final format of stats
            int yNo = 0; // yes tag no
            int nNo = 0; // no tag no
            int iNo = 0; // i tag no
            int oNo = 0; // other tag no
            // go through all lines
            while((line = reader.readLine()) != null)
            {
                lineNo++;
                if(line.startsWith("#") == false)
                {
                    // print out report
                    if(line.endsWith("|Y") == true)	// base form
                    {
                        yNo++;
                        yWriter.write(line);
                        yWriter.newLine();
                    }
                    else if(line.endsWith("|I") == true)	// inflections
                    {
                        iNo++;
                        iWriter.write(line);
                        iWriter.newLine();
                        // print expansion and POS => for not base form
                        String outStr = GetExpansionPos(line);
                        notBaseWriter.write(outStr);
                        notBaseWriter.newLine();
                    }
                    else if(line.endsWith("|N") == true)
                    {
                        nNo++;
                        nWriter.write(line);
                        nWriter.newLine();
                        // print expansion and POS => for not base form
                        String outStr = GetExpansionPos(line);
                        notBaseWriter.write(outStr);
                        notBaseWriter.newLine();
                        // print expansion and POS => for not LMW
                        notLmwWriter.write(outStr);
                        notLmwWriter.newLine();
                    }
                    else
                    {
                        oNo++;
                        oWriter.write(line);
                        oWriter.newLine();
                    }
                }
            }
            // close
            reader.close();
            yWriter.close();
            nWriter.close();
            oWriter.close();
            iWriter.close();
            notLmwWriter.close();
            notBaseWriter.close();
            // print out stats
            System.out.println("- lineNo: " + lineNo);
            System.out.println("- Y tag No: " + yNo);
            System.out.println("- N tag No: " + nNo);
            System.out.println("- I tag No: " + iNo);
            System.out.println("- Other tag No: " + oNo + " (should be 0)");
        }
        catch(Exception x)
        {
            System.out.println("** ERR@ " + lineNo + " :[" + line + "] "
                + x.toString());
        }
    }
    private static String GetExpansionPos(String inStr)
    {
        // 1. find expansion between " - new EUI (" and " - New): @ [E");
        String expStartStr = " - new EUI (";
        String expEndStr = " - New): @ [";
        int expStartStrSize = expStartStr.length();
        int expEndStrSize = expEndStr.length();
        int expStartIndex = inStr.indexOf(expStartStr) + expStartStrSize;
        int expEndIndex = inStr.indexOf(expEndStr);
        String expansion = inStr.substring(expStartIndex, expEndIndex);
        // 2. find POS, expansion must have same POS
        // some expansion has right spelling but illegal POS
        String posEndStr = "] => Manually add to candidate list";
        int posEndIndex = inStr.indexOf(posEndStr);
        // format: EUI|acr|pos => EUI:8, |: 1, => 9 
		int offset = 9;
        int posStartIndex = inStr.indexOf(SP, 
			expEndIndex + offset + expEndStrSize);
        String pos = inStr.substring(posStartIndex+1, posEndIndex);
        String outStr = expansion + SP + pos;
        return outStr;
    }
    // private method
    private static final String SP = "|";
}
